# hobo - portal to configure and deploy applications
# Copyright (C) 2015-2024 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.conf import settings
from rest_framework import permissions
from rest_framework.exceptions import PermissionDenied

from .rest_authentication import APIClientUser


class PermissionContextError(PermissionDenied):
    pass


class APIClientBasePermission:
    @staticmethod
    def is_apiclient(user):
        return isinstance(user, APIClientUser)

    @staticmethod
    def this_service():
        for services in settings.KNOWN_SERVICES.values():
            for service in services.values():
                if service.get('this', False):
                    return service
        raise PermissionContextError('No active service')

    @classmethod
    def is_service_superuser(cls, user):
        service = cls.this_service()
        ou_slug = service.get('variables', {}).get('ou-slug', None)

        if not ou_slug:
            if not user.service_superuser:
                return False
            if len(user.service_superuser.keys()) > 1:
                raise PermissionContextError('Multiple OU in permissions but no OU defined in service')
            ou_slug = list(user.service_superuser.keys())[0]

        service_superuser = user.service_superuser.get(ou_slug, {})
        if 'slug' not in service or not service['slug']:
            return False
        is_service_superuser = service_superuser.get(service['slug'], False)
        return is_service_superuser is True


class IsAPIClient(APIClientBasePermission, permissions.BasePermission):
    def has_permission(self, request, view):
        return self.is_apiclient(request.user)


class IsAdminUser(APIClientBasePermission, permissions.IsAdminUser):
    def has_permission(self, request, view):
        if self.is_apiclient(request.user):
            if self.is_service_superuser(request.user):
                return True
        else:
            return super().has_permission(request, view)
